.\"
.\" ${TBL} thisfile | ${TROFF} ${MSMACROS} | ${PRINTER}
.\"
.\" revision date - change whenever this file is edited
.ds RD 29 May 1992
.nr PO 1.2i	\" page offset 1.2 inches
.nr PD .7v	\" inter-paragraph distance
.\"
.EH 'troffcvt output'- % -''
.OH ''- % -'troffcvt output'
.OF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.EF 'Revision date:\0\0\*(RD''Printed:\0\0\n(dy \*(MO 19\n(yr'
.\"
.\" I - italic font (taken from -ms and changed)
.de I
.nr PQ \\n(.f
.if \\n(.$=0 .ft 2
.if \\n(.$>0 \&\\$3\\f2\\$1\\fP\&\\$2
..
.\"
.\" B - bold font (taken from -ms and changed)
.de B
.nr PQ \\n(.f
.if \\n(.$=0 .ft 3
.if \\n(.$>0 \&\\$3\\f3\\$1\\fP\&\\$2
..
.de St	\" troffcvt special text
\\&\\$3\fB@\\$1\fR\\$2
..
.de Cl	\" troffcvt control
\\&\\$3\fB\e\\$1\fR\\$2
..
.de Rq	\" troff request
\\&\\$3\fB\.\\$1\fR\\$2
..
.de Es	\" troff escape
\\&\\$3\fB\e\\$1\fR\\$2
..
.de Co	\" control output
.LP
\(bu \fB\e\\$1\fR \\$2 \\$3 \\$4 \\$5
.ft R
.br
..
.TL
troffcvt Output Format
.sp .3v
and
.sp .3v
Postprocessor Writing
.AU
Paul DuBois
dubois@primate.wisc.edu
.AI
Wisconsin Regional Primate Research Center
Revision date:\0\0\*(RD
.NH
Introduction
.LP
.I troffcvt
turns
.I troff
input files into a more easily parsed intermediate format to assist in
the process of developing
.I troff -to-XXX
translators.
To further assist in this process, the
.I troffcvt
distribution contains code implementing a standard reader that sequences
.I troffcvt
output into tokens.
This allows some of the job of writing postprocessors to be automated.
The combination of
.I troffcvt
and the
.I troffcvt
reader essentially turns
.I troff
files into a typed token stream.
.LP
This document describes
.I troffcvt
output format and discusses how to write postprocessors that convert
such output into some target format.
If you decide not to use the reader when writing a postprocessor, you
must understand how to interpret
.I troffcvt
files.
If you do use the reader, then
you don't need to know much about
.I troffcvt
format since the reader tokenizes everything for you.
However, it's still useful to have at least a rudimentary
knowledge of the format.
.NH
Output format
.LP
.I troffcvt
writes three kinds of lines:
.IP (i)
Control lines used to control text processing.
These begin with a backslash ``\fB\e\fR'' and a keyword, and may have
arguments separated by spaces following the keyword.
Control lines pertain to document structure.
.IP (ii)
Plain text lines consisting of literal text to be written without further
interpretation.
Together with special text lines, plain text lines comprise the content
of a document.
.IP (iii)
Special text lines used to indicate special characters.
These begin with an at-sign ``\fB@\fR'' followed by
a special character name, e.g.,
.St grave-accent
.B \` ), (
.St ldblquote
.B `` ), (
.St alpha
.B \(*a ). (
The special character should be mapped to a character appropriate for the
target format.
.LP
A
.I troffcvt
output file is structured as follows:
.LP
.DS
.ft B
\ebegin-setup
\eresolution\fI N
other setup lines...
.ft B
\eend-setup
.ft I
rest of document...
.ft R
.DE
.LP
The first portion of the file consists of
a setup section bracketed by
.Cl begin-setup
and
.Cl end-setup
lines.
The lines in between indicate the initial document layout.
The first line of this information is
.Cl resolution
.I N ,
where
.I N
is the number of basic units per inch.
This indicates the resolution at which
.I troffcvt
performed its calculations.
Numbers obtained from other control lines may be converted to
ems, points, etc. using this resolution.
For instance, if the resolution is 1440, the control line
.Cl spacing
.B 240
indicates a baseline spacing of 1/6 inch.
.LP
The default resolution used by
.I troffcvt
is 432, but can be changed to whatever you want.
Probably the
.I lowest
resolution you want to use is the least common multiple of 72 and the
resolution you expect to use in the target format.
Otherwise you may end up with ugly round-off errors when you convert units
back to ems, points, etc.
.LP
If the resolution is
.I r ,
other common
.I troff
units may be calculated as follows.
.I S "" (
is the current point size.)
.LP
.TS
center box;
cfB | cfB | lfB
c|l|l.
Unit	Name	Number of basic units
_
\fBi\fR	Inch	\fIr\fR
\fBc\fR	Centimeter	\fIr\fR\^\(mu\^50\(sl127
\fBP\fR	Pica = 1\(sl6 inch	\fIr\fR/6
\fBm\fR	Em = \fIS\fR points	\fIS\fR\^\(mu\^\fIr\fR/72\^=\^\fISr\fR/72
\fBn\fR	En = Em\(sl2	\fIS\fR\^\(mu\^\fIr\fR/72\^\(mu\^1/2\^=\^\fISr\fR/144
\fBp\fR	Point = 1\(sl72 inch	\fIr\fR/72
\fBu\fR	Basic unit	1
\fBv\fR	Vertical line space	varies; set by \fB\espacing\fR \fIN\fR
.TE
.LP
The rest of the lines in the setup section contain information for the
page length, page width, indents, etc.
.NH
Special Text Lines
.LP
.I troffcvt
knows the names of special characters from two sources of information.
The input sequence and output sequence for a given special character
is either built in and recognized
implicitly, or taken from the action file read at runtime.
Special characters with an input sequence of the
form \fB\e(\fIxx\fR are always in the latter category.
.NH 2
Built-in Special Characters
.LP
Built-in characters form a short list.
Most of these are in the ``Escape Sequences for Characters, Indicators, and Functions''
section of the
.I troff
.B "Summary and Index" .
.LP
.TS
center box;
cfB | cfB | cfB
lfB | l | l.
Input Sequence	Output Sequence	Note
_
\ee	@backslash	affected by \fB.ec\fR
\e&	@zero-space
\e^	@twelfth-space
\e|	@sixth-space
\e0	@digit-space
\e(space)	@hard-space
\e\-	@minus
\e\'	@acute-accent
\e\`	@grave-accent
\e%	@opt-hyphen	affected by \fB.hc\fR
\ea\fR,\fISOH\fR	@leader
\et\fR,\fITAB\fR	@tab
\e(backspace)	@backspace
\`\`	@ldblquote
\'\'	@rdblquote
\`	@lsglquote
\'	@rsglquote
.TE
.LP
The output sequence for
.Es e
actually depends on the current escape character, which may be changed with
.Rq ec .
The input sequence for the optional hyphenation character may be changed
with
.Rq hc .
.LP
The characters
.B @
and
.B \e
have special meaning in
.I troffcvt
files, so they are indicated as specials
.St at
and
.St backslash
where they are to appear in
the final output literally.
Postprocessors should convert them back to
.B @
and
.B \e .
.LP
The field delimiter and field-pad characters defined with
.Rq fc
are written out as
.St begin-field
or
.St end-field ,
and
.St field-pad .
(Odd delimiters begin fields; even ones end fields).
.NH 2
Non-built-in Special Characters
.LP
The number of non-built-in characters is not fixed.
All the special characters listed in the Ossanna manual are defined in
the default action file supplied with the
.I troffcvt
distribution, but the list may be modified as necessary to reflect
extra special characters available in
your local version(s) of
.I troff .
.NH
The troffcvt Reader
.LP
The
.I troffcvt
file reader reads
.I troffcvt
output and tokenizes it, setting several global variables in the process:
.LP
.DS
.ta 1.1i
\fItcrClass\fR	token class
\fItcrMajor\fR	token major number
\fItcrMinor\fR	token minor number
\fItcrArgv\fR[\|]	token text vector
\fItcrArgc\fR	number of elements in \fItcrArgv\fR[\|] vector
.DE
.LP
All tokens are assigned to a class.
The other variables are set or not depending on the class.
The classes are:
.LP
.DS
.ta 1.1i
\fItcrEOF\fR	end of input
\fItcrControl\fR	control line
\fItcrText\fR	plain text character
\fItcrSText\fR	special text character
.DE
.LP
Elements of
.I tcrArgv [\|]
are null-terminated strings.
There are
.I tcrArgc
elements in the vector.
For plain text and special text tokens,
.I tcrArgc
is always 1.
For control tokens, the rest of the line is automatically
parsed to find any following arguments and these are placed into
.I tcrArgv [1]
through
.I tcrArgv [\fItcrArgc\fR\-1].
.I tcrArgv [\fItcrArgc\fR]
is NULL in all cases.
.LP
All numbers on control lines are written as integers.
A string representing a number may be converted to
.I long
with
.I TCRStrToNum() .
.LP
To use the reader, call
.I TCRInit()
first.
Then call
.I TCRGetToken()
repeatedly.
The token class
.I tcrClass
is the return value of
.I TCRGetToken() .
When the reader returns
.I tcrEOF ,
the postprocessor should finish up.
.LP
Here is how the global variables are set for the various token classes.
.LP
\fItcrClass = tcrEOF\fR:
.DS
none of the other variables are set.
.DE
.LP
\fItcrClass = tcrControl\fR:
.DS
.ta 1.1i
\fItcrMajor\fR	control major number (see \fItcr.h\fR)
\fItcrMinor\fR	major number subtype (not set for all control words, see \fItcr.h\fR)
\fItcrArgv\fR[0]	text of control word, including leading \e
\fItcrArgv\fR[>0]	arguments following control word
\fItcrArgc\fR	number of arguments, including control word
.DE
.LP
\fItcrClass = tcrText\fR:
.DS
.ta 1.1i
\fItcrMajor\fR	ASCII value of character (each character is a separate token)
\fItcrArgv\fR[0]	one-byte string containing the character
\fItcrArgc\fR	= 1
.DE
.LP
\fItcrClass = tcrSText\fR:
.DS
.ta 1.1i
\fItcrMajor\fR	usually \fItcrSTUnknown\fR, but see below
\fItcrArgv\fR[0]	text of special character name, including leading @
\fItcrArgc\fR	= 1
.DE
All the built-in special characters are recognized and assigned distinct
major numbers.
Other specials are assigned the major number
.I tcrSTUnknown
and the postprocessor must
examine the text of the token
.I tcrArgv [0]) (
to determine what it is and what to do with it.
There is no way for the reader to assign fixed numbers to these since the
set of special characters understood by
.I troffcvt
isn't fixed.
One way of dealing with the problem is to
read at runtime a file of all the special character
names you expect to see.
(Usually the same set of names specified in the action file used with
.I troffcvt .)
.LP
Note:
although built-in special characters
.I do
have fixed major numbers assigned, there is nothing to prevent you
from processing them like other specials, i.e., by examining the token text.
It may be more convenient to treat all specials uniformly.
.LP
The reader changes the characteristics of the default scanner.
This is done in
.I TCRInit() .
If you use the token scanning library for other purposes in your application,
you need to change the scanner's characteristics to what you want, and then
restore them or
.I TCRGetToken()
may not work correctly.
.NH
Postprocessors
.LP
I assume in this section that you use the
.I troffcvt
reader to write a postprocessor.
It's not necessary that you do so, but if you don't, most of the following
comments don't apply.
.LP
It's best if you examine the source for some of the postprocessors supplied
in the
.I troffcvt
distribution before trying to write one of your own.
You should also read ``\fItroffcvt \*- Notes, Bugs, Deficiences\fR'' to
acquaint yourself with
.I troffcvt 's
idiosyncrasies.
.LP
A postprocessor can be set up this way:
.IP (i)
Call the reader to read tokens until
.Cl end-setup
is seen,
processing control lines to initialize the document
layout state.
.IP (ii)
Dump out initial layout information (if necessary) in a form appropriate
to the target format.
.IP (iii)
Call the reader to read the rest of the document until it returns
.I tcrEOF .
Process each token as it is returned:
.RS
.IP \(bu
Plain text \*- write to output, except that
end of line character(s) should be ignored (CR, LF or CRLF, depending
on the convention used on your system).
.IP \(bu
Special text \*-
the special character should be mapped to a character appropriate for the
target format and written out.
If no exact equivalent is available, something reasonably close should be substituted.
For instance,
.St ldblquote
denotes double left curly quotes.
If the output format only has straight quotes (\fB"\fR), that
might be a reasonable substitute.
.IP
In extreme cases, no reasonable substitute exists.
Some of the translators supplied in the distribution adopt the convention of writing
``[[\fIcharacter-name\fR]]'' when this happens, in order that the
fact of the character's having occurred in the document not be lost.
(Since it is up to the user to notice failures of translation and take
appropriate action, it is more likely that such notice will be taken if an
obvious sequence is written rather than if the character is simply dropped.)
.IP \(bu
Control tokens \*- modify document processing state.
.RE
.LP
The above procedure
may seem deceptively simple, and, considered in the abstract, it
is.
I find it easiest to begin with a copy of
.I tc2null.c ,
which simply routes tokens through a bunch of switch statements.
The switches can be filled in as you decide what to do
with various tokens, which allows incremental development of a postprocessor.
You'll probably find that, although this approach is conceptually simple, in
practice the details can quickly become more complex than you'd like.
Some of the important issues about which you need to be concerned are
discussed in the following sections.
.NH 2
Font Handling.
.LP
How do
.I troff
fonts correspond to fonts available in your target format?
Remember that you cannot predict in advance which fonts might be used in
a
.I troff
document, although you can probably determine which ones are available
at your site.
.LP
The reader supplies a certain amount of convenience machinery for handling
fonts in
.I troffcvt
documents.
Treatment of fonts in
.I troff
doesn't correspond well to other document formats (at least in my experience).
In
.I troff
one switches from plain text to italic or boldface by switching fonts,
e.g., from
.B R
to
.B I ,
or from
.B R
to
.B B .
In other formats this is accomplished by leaving the typeface the same but
applying different style attributes to it.
It is evident that
.I troff
collapses the two dimensions of typeface and style onto a single-dimensional
font namespace.
.LP
For purposes of font support in the
.I troffcvt
reader
it is usually more fruitful to map
.I troff
font names back onto typeface-style pairs, where the typeface is the
font family a given font derives from and the style indicates those
attributes that need to be applied to the plain font in that
family to produce the effect of the
.I troff
font.
For instance, the default
.I troff
fonts
.B R ,
.B I
and
.B B
can be described as follows:
.LP
.TS
center box;
lfB | lfB | lfB
l | l | l .
Font	Typeface	Style
_
R	Times
I	Times	italic
B	Times	bold
.TE
.LP
Treating fonts this way allows
.I troff
fonts to be manipulated so that ``font'' changes that really
correspond to style changes can be handled as such.
.LP
A simple font to typeface-style map is included in the distribution
(the
.I tcr-fonts
file).
This file should be modified as necessary to reflect fonts available locally
at your site and installed into
.I /usr/lib/troffcvt .
.I r-font.c
contains the code to use the font map.
The sample postprocessor
.I tc2rtf.c
shows how to use the routines in
.I r-font.c .
This font mechanism is simple and may not apply to your situation.
If it doesn't, don't use it.
.LP
I suppose another approach might be to intercalate a ``mid-processor''
(e.g., a
.I sed
script), between
.I troffcvt
and the postprocessor, to look for
.Cl font
control lines and map them onto fonts more easily understood by the
postprocessor.
.NH 2
Special Character Handling
.LP
How will you specify what to do with special characters?
Remember that the reader assigns distinct major numbers only to those
special characters for which recognition is built into
.I troffcvt .
.LP
Generally, postprocessors read in a list of special characters
that parallels the list given in the action file used by
.I troffcvt .
If the action file list is changed, the postprocessor's list should be,
too.
This is a headache, but at least the changes can be made by editing text files
rather than by recompiling programs.
.NH 2
Text Filling, Adjusting and Centering
.LP
Text filling, adjusting and centering interact in
.I troff .
My understanding of how this works is indicated below.
Since my conceptual scheme is instantiated in the code,
let's hope it's correct.
.LP
Centering
.Rq ce ) (
takes precedence over filling and adjustment.
When centering is not on, no-fill mode
.Rq nf ) (
suspends filling and adjustment; input lines are simply copied to the
output, left justified.
If centering is off and filling is on
.Rq fi ), (
input lines are joined
as necessary to fill output lines, which are then adjusted according to
the current adjustment specified by
.Rq ad .
Adjustment may be suspended with
.Rq na .
.LP
Turning off filling does not change the adjustment setting; adjustment
is merely suspended while filling is off, and goes back into effect
when filling is turned back on.
Similarly, centering doesn't change the filling or adjustment settings;
they are suspended while centering is in effect and resume when centering
terminates.
.LP
.I troffcvt
removes the need for postprocessors to handle these centering, filling
and adjusting (CFA) interactions, by always expliciting writing
out which CFA control code to use.
This means the postprocessor only need remember the most recent one.
If
.I troffcvt
did not do this, postprocessors would need to maintain a bunch of
state variables (currently centering?
currently filling?
currently adjusting?
which type of adjustment?).
.LP
Centering is turned on with
.Cl center .
All text up to a
.Cl break
should be placed on a single output line and centered.
.LP
No-fill is turned on with
.Cl nofill .
All text up to a
.Cl break
should be placed on a single output line,
left-justified.
.LP
If neither centering nor no-fill are in effect, filling is on and
one of the adjustment modes
.Cl adjust-left ,
.Cl adjust-right ,
.Cl adjust-full
or
.Cl adjust-center
will be issued.
All text up to the next
.Cl break
should be used to fill output lines.
All of these except the last in a paragraph should be adjusted
in the proper way.
.LP
Postprocessors can likely treat
.Cl center
and
.Cl adjust-center
as equivalent.
Ditto for
.Cl nofill
and
.Cl adjust-left .
The reason for this is that
.I troffcvt
will have inserted extra
.Cl break
lines where necessary for the first mode of each pair.
.LP
Note that there are no control words such as
.Cl nocenter ,
.Cl fill
or
.Cl noadjust .
Centering is turned off by
.Cl nofill
and the adjustment indicators.
Filling is turned on by the adjustment indicators.
The
.I troff
no-adjust request
.Rq na
seems functionally equivalent to left-adjustment and so is indicated
with
.Cl adjust-left .
The reason for the
.Rq na
request seems to be so that
.Rq na
can be followed by
.Rq ad
(with no argument) to resume whatever adjustment mode was in effect
prior to the
.Rq na .
Since
.I troffcvt
keeps track of adjustment modes it can write out the proper indicator
explicitly.
.LP
It is not the case that
.I troffcvt
output will contain a single line of text corresponding to each input
line when no-fill or centering are in effect.
When input contains special characters, each of these is written to their
own output line, for instance.
This illustrates why it's important to read text until a
.Cl break
is seen.
.NH 2
Paragraphing
.LP
Paragraph text should be treated conceptually as one unbroken string of text,
even though it may appear physically on several lines of
.I troffcvt
output.
Thus, successive text lines (either plain or special) should
be considered to be part of the same paragraph until a
.Cl break
control line occurs.
The postprocessor should perform line filling and wrapping according to
the most recent centering, filling or adjustment control line (one of
.Cl center ,
.Cl nofill ,
.Cl justify-left ,
.Cl justify-right ,
.Cl justify-full
or
.Cl justify-center ).
.LP
All characters on plain text lines are significant except the terminating
linefeed, which should be ignored.
Postprocessors should not strip leading or trailing
spaces.
Postprocessors also should not insert space characters between successive
text lines; where necessary, spaces will already
have been placed within the text itself.
One exception is that the
decision as to whether to put one or two spaces between sentences
is left to the postprocessor.
The main difficulty is determining when a sentence ends.
If the usual suggested style for creation of
.I troff
input files is followed (i.e., that each sentence should begin on a new line),
sentence-terminating periods, question marks and exclamation
points will occur
at the ends of lines.
This property is preserved in
.I troffcvt
output.
Postprocessors thus can locate sentence endings and
have the information they need for determining whether
to insert extra spaces, should they wish to do so.
.LP
With the exception of input line gluing resulting from stripping
embedded newlines, input lines are never joined together in the output.
In contrast, input lines are often split up in the output.
For instance, the input ``a\e\e\efBb\efR\e(*ac\e0d'' is turned into several
output lines:
.LP
.DS
.ta 1.5i
a
@backslash
\efont B
b
\efont R
@alpha
c
@digit-space
d
.DE
.LP
Some document formats indicate paragraphs when they begin,
others when they end.
The postprocessor will need to follow whichever convention is
used in the target format.
This should be a simple matter since
paragraph beginnings and endings both are readily located in
.I troffcvt
output.
.B \ebreak
corresponds to paragraph endings.
Beginnings are easily found also:
the first text line begins one, and every time a
.B \ebreak
occurs, the following text line begins one.
(Remember that there may be other non-text lines between the
.B \ebreak
and the following text line, though.)
.NH 2
Miscellaneous
.LP
.B Tabs.
Tab stops should be interpreted relative to the current indent,
.I not
the page offset.
This means that if tab stops are set and then the indent is changed,
the effective tab stops relative to the page offset change.
Some
postprocessors may need to reset tabs in the target format
when that happens.
.NH
Control Line Reference
.LP
The section documents the syntax of all control lines produced by
.I troffcvt .
Numeric values on control lines are usually, but not always, specified
in basic units.
The descriptions are grouped according to the section of the
Ossanna
.I troff
manual to which they are most closely related.
.SH
\(sc 0. Miscellaneous
.Co begin-setup
.IP
Indicates the beginning of the setup section of
.I troffcvt
output.
.Co end-setup
.IP
Indicates the end of the setup section of
.I troffcvt
output.
When it occurs, the basic layout of the document will have been specified.
.Co resolution \fIN\fR
.IP
This line occurs first within the setup section.
It indicates the resolution at which
.I troffcvt
performed its calculations, in basic units per inch.
.Co comment \fIstring\fR
.IP
Indicates a comment, which may be ignored.
The entire
.I string
will be in
.I tcrArgv [1].
Note that
.Cl comment
may appear at any time, even before
.Cl begin-setup .
.SH
\(sc 2. Fonts and Character Size Control
.Co font \fIF\fR
.IP
Switch to font
.I F .
.Co constant-width \fIF\fR
.IP
Umm...
.Co noconstant-width \fIF\fR
.IP
Umm...
.Co embolden \fIF\fR N
.IP
Embolden font \fIF\fR by smearing it
.I N
units.
If
.I N
is zero, emboldening should be turned off.
.IP
If you implement emboldening,
you might find it more profitable to ignore the smear value and make the
font bold by some means other than reprinting the characters slightly
displaced from the original printing like
.I troff
does.
.Co embolden-special \fIF\fR N
.IP
Like
.Cl embolden ,
but embolden characters in the special font whenever the current font is
.I F .
In practice this probably has no meaning for most postprocessors,
because the special font in
.I troff
is logically part of the three default fonts, something unlikely to be
true in the target output format.
.Co point-size \fIN\fR
.IP
Set point size to
.I N .
.I N
is number of points, not basic units.
.Co space-size \fIN\fR
.IP
Set space size to
.I N /36\fBm\fR.
Note that the actual instantaneous value thus depends of the size of an
.B em
(and thus on the current point size).
For this reason it may be best to maintain
.I N
but also recompute the actual space size in basic units whenever the point
size changes.
.SH
\(sc 3. Page Control
.Co begin-page [\fIN\fR]
.IP
If
.I N
is present, begin new page numbered
.I N .
Otherwise, just begin new page (presumably numbered in sequence with the
current page).
.Co offset \fIN\fR
.IP
Set page offset to
.I N .
.Co page-length \fIN\fR
.IP
Set page length to
.I N .
.Co page-number \fIN\fR
.IP
Set page number to
.I N .
(Does not begin a new page.)
.Co need \fIN\fR
.IP
.I N
units of vertical space are needed.
If less than that remains on the current page, begin a new page.
.Co mark
.IP
Umm...
.SH
\(sc 4. Text Filling, Adjusting, and Centering
.Co adjust-center
.Co adjust-full
.Co adjust-left
.Co adjust-right
.IP
Fill output lines, adjusting as indicated.
.Co nofill
.IP
Do not fill output lines.
.Co center
.IP
Center output lines.
.Co break
.IP
End of input line.
Flush and terminate current output line.
.Co break-spread
.IP
Break at end of current word and spread output line to current line length.
.SH
\(sc 5. Vertical Spacing
.Co spacing \fIN\fR
.IP
Set vertical base-line spacing to
.I N
units, which becomes the meaning of 1
.B v .
.Co line-spacing \fIN\fR
.IP
Set line spacing to
.I N
.B v 's.
.Co space \fIN\fR
.IP
Space vertically
.I N
units (negative = upward).
.Co extra-space \fIN\fR
.IP
Umm...
.SH
\(sc 6. Line Length and Indenting
.Co indent \fIN\fR
.IP
Set indent to
.I N
units.
.Co line-length \fIN\fR
.IP
Set line length to
.I N
units.
.Co temp-indent \fIN\fR
.IP
Set temporary indent to
.I N
units.
In
.I troff ,
this turns off after the next output line.
In other formats it likely corresponds to ``paragraph first line indent'' or
something similar.
.SH
\(sc 7. Macros, Strings, Diversions, and Position Traps
.Co begin-diversion \fIname\fR
.IP
Indicates that following output, until
.Cl end-diversion
.I name
occurs, was intended to be diverted to macro
.I xx .
.Co append-diversion \fIname\fR
.IP
Indicates that following output, until
.Cl end-diversion
.I name
occurs, was intended to be appended to diversion macro
.I xx .
.Co end-diversion \fIname\fR
.IP
Indicates end of preceding
.Cl begin-diversion
.I name
or
.Cl append-diversion
.I name .
.SH
\(sc 9. Tabs, Leaders, and Fields
.Co reset-tabs
.IP
Reset tab stops to default (every half-inch).
.Co first-tab \fIN c\fR
.IP
Clear tab stops and install first one at
.I N
units.
.I c
is
.B l
(left),
.B c
(center),
or
.B r
(right).
.Co next-tab \fIN c\fR
.IP
Add tab stop to current set.
.I N
and
.I c
are as for
.Cl first-tab .
.Co tab-char [\fIc\fR]
.IP
Set tab repetition character to
.I c .
If
.I c
is missing, tabs should be implemented as motion.
.Co leader-char [\fIc\fR]
.IP
Set leader repetition character to
.I c .
If
.I c
is missing, leaders should be implemented as motion.
.SH
\(sc 10. Input and Output Conventions and Character Translations
.Co underline
.IP
Turn on underlining.
.Co cunderline
.IP
Turn on continuous underlining.
.Co nounderline
.IP
Turn off underlining (both kinds).
.Co underline-font \fIF\fR
.IP
Set underline font to
.I F .
.SH
\(sc 11. Local Horizontal and Vertical Motions, and the Width Function
.Co motion \fIN c\fR
.IP
Move
.I N
units.
.I c
is
.B h
for horizontal motion (negative = left) or
.B v
vertical motion (negative =
upward).
.Co line \fIN c\fR
.IP
Draw line.
.I N
and
.I c
are as for
.Cl motion .
.SH
\(sc 12. Overstrike, Bracket, Line-drawing, and Zero-width Functions
.Co begin-bracket
.IP
Characters on following text lines
(until
.Cl end-bracket )
should be used to build a bracket.
.Co end-bracket
.IP
Terminates preceding
.Cl begin-bracket .
.Co begin-overstrike
.IP
Characters on following text lines
(until
.Cl end-overstrike )
should be overstruck.
.Co end-overstrike
.IP
Terminates preceding
.Cl begin-overstrike .
.Co zero-width \fIc\fR
.IP
Print
.I c
without changing position on page.
.SH
\(sc 13. Hyphenation
.Co hyphenate \fIN\fR
.IP
Set hyphenation mode.
Zero = off; non-zero is interpreted as in
.I troff
manual.
.SH
\(sc 14. Three Part Titles
.Co title-length \fIN\fR
.IP
Set title length to
.I N
units.
.Co begin-title name \fIc\fR
.IP
Following output, until
.Cl end-title
.I name
is part of title.
.I c
is
.B l
(left),
.B c
(center),
or
.B r
(right).
.Co end-title
.IP
Terminates preceding
.Cl begin-title .
